@@ -1,10 +1,10 @@
-// Copyright (C) 2016 The Qt Company Ltd.
+// Copyright (C) 2023 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
 
 #include "statemachine.h"
 #include "../trafficlight-common/trafficlight.h"
 
-#include <QApplication>
+#include <QtWidgets/qapplication.h>
 
 int main(int argc, char **argv)
 {
