@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtScxml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest>
 #include <QtTest/private/qpropertytesthelper_p.h>
@@ -56,6 +31,7 @@ private Q_SLOTS:
 
     void doneDotStateEvent();
     void running();
+    void restart();
 
     void invokeStateMachine();
 
@@ -63,8 +39,18 @@ private Q_SLOTS:
     void logWithoutExpr();
 
     void bindings();
+
+    void setTableDataUpdatesObjectNames();
+    void errorsFromNestedStatemachine(); // QTBUG-135396
 };
 
+void tst_StateMachine::errorsFromNestedStatemachine()
+{
+    QScopedPointer<QScxmlStateMachine> machine(QScxmlStateMachine::fromFile(":/tst_statemachine/nestedinvoker.scxml"));
+    auto error = machine->parseErrors().front();
+    QCOMPARE(error.fileName(), "nestederror.scxml");
+}
+
 void tst_StateMachine::stateNames_data()
 {
     QTest::addColumn<QString>("scxmlFileName");
@@ -98,7 +84,7 @@ void tst_StateMachine::stateNames()
 
     QScopedPointer<QScxmlStateMachine> stateMachine(QScxmlStateMachine::fromFile(scxmlFileName));
     QVERIFY(!stateMachine.isNull());
-    QCOMPARE(stateMachine->parseErrors().count(), 0);
+    QCOMPARE(stateMachine->parseErrors().size(), 0);
 
     QCOMPARE(stateMachine->stateNames(compressed), expectedStates);
 }
@@ -389,7 +375,7 @@ void tst_StateMachine::doneDotStateEvent()
 
     stateMachine->start();
     finishedSpy.wait(5000);
-    QCOMPARE(finishedSpy.count(), 1);
+    QCOMPARE(finishedSpy.size(), 1);
     QCOMPARE(stateMachine->activeStateNames(true).size(), 1);
     QVERIFY(stateMachine->activeStateNames(true).contains(QLatin1String("success")));
 }
@@ -406,15 +392,33 @@ void tst_StateMachine::running()
 
     stateMachine->start();
 
-    QCOMPARE(runningChangedSpy.count(), 1);
+    QCOMPARE(runningChangedSpy.size(), 1);
     QCOMPARE(stateMachine->isRunning(), true);
 
     stateMachine->stop();
 
-    QCOMPARE(runningChangedSpy.count(), 2);
+    QCOMPARE(runningChangedSpy.size(), 2);
     QCOMPARE(stateMachine->isRunning(), false);
 }
 
+void tst_StateMachine::restart()
+{
+    QScopedPointer<QScxmlStateMachine> stateMachine(
+            QScxmlStateMachine::fromFile(QString(":/tst_statemachine/stateDotDoneEvent.scxml")));
+    QVERIFY(!stateMachine.isNull());
+
+    QSignalSpy finishedSpy(stateMachine.data(), SIGNAL(finished()));
+
+    stateMachine->start();
+    finishedSpy.wait(5000);
+    QCOMPARE(finishedSpy.size(), 1);
+    QCOMPARE(stateMachine->activeStateNames(true).size(), 1);
+    QVERIFY(stateMachine->activeStateNames(true).contains(QLatin1String("success")));
+
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("(.*)Can't start finished machine"));
+    stateMachine->start();
+}
+
 void tst_StateMachine::invokeStateMachine()
 {
     QScopedPointer<QScxmlStateMachine> stateMachine(
@@ -426,7 +430,7 @@ void tst_StateMachine::invokeStateMachine()
     QTRY_VERIFY(stateMachine->activeStateNames().contains(QString("anyplace")));
 
     QList<QScxmlInvokableService *> services = stateMachine->invokedServices();
-    QCOMPARE(services.length(), 1);
+    QCOMPARE(services.size(), 1);
     QVariant subMachineVariant = services[0]->property("stateMachine");
     QVERIFY(subMachineVariant.isValid());
     QScxmlStateMachine *subMachine = qvariant_cast<QScxmlStateMachine *>(subMachineVariant);
@@ -445,7 +449,7 @@ void tst_StateMachine::multipleInvokableServices()
     QCOMPARE(stateMachine->isRunning(), true);
 
     finishedSpy.wait(5000);
-    QCOMPARE(finishedSpy.count(), 1);
+    QCOMPARE(finishedSpy.size(), 1);
     QCOMPARE(stateMachine->activeStateNames(true).size(), 1);
     QVERIFY(stateMachine->activeStateNames(true).contains(QLatin1String("success")));
 }
@@ -458,7 +462,7 @@ void tst_StateMachine::logWithoutExpr()
     QTest::ignoreMessage(QtDebugMsg, "\"Hi2\" : \"\"");
     stateMachine->start();
     QSignalSpy logSpy(stateMachine.data(), SIGNAL(log(QString,QString)));
-    QTRY_COMPARE(logSpy.count(), 1);
+    QTRY_COMPARE(logSpy.size(), 1);
 }
 
 void tst_StateMachine::bindings()
@@ -474,11 +478,16 @@ void tst_StateMachine::bindings()
         return;
     }
 
+    using StateMachinePtr = std::unique_ptr<QScxmlStateMachine>;
+
     // -- QScxmlStateMachine::initialValues
     QVariantMap map1{{"map", 1}};
     QVariantMap map2{{"map", 2}};
     QTestPrivate::testReadWritePropertyBasics<QScxmlStateMachine, QVariantMap>(
-                *stateMachine1, map1, map2, "initialValues");
+            *stateMachine1, map1, map2, "initialValues",
+            []() {
+                return StateMachinePtr{QScxmlStateMachine::fromFile(QString("not_a_real_file"))};
+            });
     if (QTest::currentTestFailed()) {
         qWarning() << "QScxmlStateMachine::initialValues bindable test failed.";
         return;
@@ -493,7 +502,10 @@ void tst_StateMachine::bindings()
     MockLoader loader1;
     MockLoader loader2;
     QTestPrivate::testReadWritePropertyBasics<QScxmlStateMachine, QScxmlCompiler::Loader*>(
-                *stateMachine1, &loader1, &loader2, "loader");
+            *stateMachine1, &loader1, &loader2, "loader",
+            []() {
+                return StateMachinePtr{QScxmlStateMachine::fromFile(QString("not_a_real_file"))};
+            });
     if (QTest::currentTestFailed()) {
         qWarning() << "QScxmlStateMachine::loader bindable test failed.";
         return;
@@ -508,18 +520,25 @@ void tst_StateMachine::bindings()
     QScxmlNullDataModel model1;
     // data can only change once
     QTestPrivate::testWriteOncePropertyBasics<QScxmlStateMachine, QScxmlDataModel*>(
-                *stateMachine2, nullptr, &model1, "dataModel");
+            *stateMachine2, nullptr, &model1, "dataModel", true,
+            []() {
+                return StateMachinePtr{QScxmlStateMachine::fromFile(QString("not_a_real_file"))};
+            });
     if (QTest::currentTestFailed()) {
         qWarning() << "QScxmlStateMachine::dataModel bindable test failed.";
         return;
     }
 
     // -- QScxmlStateMachine::tableData
-    // Use the statemachine to generate the tabledDatas for testing
+    // Use the statemachine to generate the tableData for testing
     std::unique_ptr<QScxmlStateMachine> stateMachine4(
                 QScxmlStateMachine::fromFile(QString(":/tst_statemachine/invoke.scxml")));
     QTestPrivate::testReadWritePropertyBasics<QScxmlStateMachine, QScxmlTableData*>(
-                *stateMachine2, stateMachine1.get()->tableData(), stateMachine4.get()->tableData(), "tableData");
+            *stateMachine2, stateMachine1.get()->tableData(), stateMachine4.get()->tableData(),
+            "tableData",
+            []() {
+                return StateMachinePtr{QScxmlStateMachine::fromFile(QString("not_a_real_file"))};
+            });
     if (QTest::currentTestFailed()) {
         qWarning() << "QScxmlStateMachine::tableData bindable test failed.";
         return;
@@ -528,32 +547,32 @@ void tst_StateMachine::bindings()
     // -- QScxmlStateMachine::invokedServices
     // Test executes statemachine and observes as the invoked services change
     TopMachine topSm;
-    QSignalSpy invokedSpy(&topSm, SIGNAL(invokedServicesChanged(const QList<QScxmlInvokableService *>)));
-    QCOMPARE(topSm.invokedServices().count(), 0);
+    QSignalSpy invokedSpy(&topSm, SIGNAL(invokedServicesChanged(QList<QScxmlInvokableService*>)));
+    QCOMPARE(topSm.invokedServices().size(), 0);
     // at some point during the topSm execution there are 3 invoked services
     topSm.start();
-    QTRY_COMPARE(topSm.invokedServices().count(), 3);
-    QCOMPARE(invokedSpy.count(), 1);
+    QTRY_COMPARE(topSm.invokedServices().size(), 3);
+    QCOMPARE(invokedSpy.size(), 1);
     // after completion invoked services drop back to 0
     QTRY_COMPARE(topSm.isRunning(), false);
-    QCOMPARE(topSm.invokedServices().count(), 0);
-    QCOMPARE(invokedSpy.count(), 2);
+    QCOMPARE(topSm.invokedServices().size(), 0);
+    QCOMPARE(invokedSpy.size(), 2);
     // bind *to* the invokedservices property and check that we observe same changes
     // during the topSm execution
     QProperty<qsizetype> invokedServicesObserver;
-    invokedServicesObserver.setBinding([&](){ return topSm.invokedServices().count(); });
+    invokedServicesObserver.setBinding([&](){ return topSm.invokedServices().size(); });
     QCOMPARE(invokedServicesObserver, 0);
     topSm.start();
     QTRY_COMPARE(invokedServicesObserver, 3);
-    QCOMPARE(topSm.invokedServices().count(), 3);
-    QCOMPARE(invokedSpy.count(), 3);
+    QCOMPARE(topSm.invokedServices().size(), 3);
+    QCOMPARE(invokedSpy.size(), 3);
 
     // -- QScxmlDataModel::stateMachine
     QScxmlNullDataModel dataModel1;
     std::unique_ptr<QScxmlStateMachine> stateMachine5(
                 QScxmlStateMachine::fromFile(QString("not_a_real_file")));
     // data can only change once
-    QTestPrivate::testWriteOncePropertyBasics<QScxmlDataModel, QScxmlStateMachine*>(
+    QTestPrivate::testWriteOncePropertyBasics<QScxmlNullDataModel, QScxmlStateMachine*>(
                 dataModel1, nullptr, stateMachine5.get(), "stateMachine");
     if (QTest::currentTestFailed()) {
         qWarning() << "QScxmlDataModel::stateMachine bindable test failed.";
@@ -561,6 +580,28 @@ void tst_StateMachine::bindings()
     }
 }
 
+void tst_StateMachine::setTableDataUpdatesObjectNames()
+{
+    std::unique_ptr<QScxmlStateMachine> stateMachine1(
+            QScxmlStateMachine::fromFile(QString(":/tst_statemachine/emptylog.scxml")));
+    const QString sm1ObjectName = stateMachine1->objectName();
+    std::unique_ptr<QScxmlStateMachine> stateMachine2(
+            QScxmlStateMachine::fromFile(QString(":/tst_statemachine/eventoccurred.scxml")));
+    const QString sm2ObjectName = stateMachine2->objectName();
+    QCOMPARE_NE(sm1ObjectName, sm2ObjectName);
+
+    std::unique_ptr<QScxmlStateMachine> sm(
+            QScxmlStateMachine::fromFile(QString("not_a_real_file")));
+    QVERIFY(sm->objectName().isEmpty());
+    // no name set, so update object name
+    sm->setTableData(stateMachine1->tableData());
+    QCOMPARE_EQ(sm->objectName(), sm1ObjectName);
+
+    // object name already set, so do not update
+    sm->setTableData(stateMachine2->tableData());
+    QCOMPARE_EQ(sm->objectName(), sm1ObjectName); // did not change
+}
+
 QTEST_MAIN(tst_StateMachine)
 
 #include "tst_statemachine.moc"
